﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace OOPBasics
{
    class Person : IDisposable
    {
        private string mName;
        private DateTime mBirthDate;
        private int mTotalDistance;
        private static int mCounter;

        public  delegate void walkDelegate(int d);
        public event walkDelegate walked;

        public Person(string name, string birthDate)
        {
            mName = name;
            mBirthDate = DateTime.Parse(birthDate);
            mCounter += 1;
        }

        public void Walk(int distance)
        {
            mTotalDistance += distance;
            OnWalked(distance);
        }

        public void Walk()
        {
            OnWalked(0);
        }

        protected void OnWalked(int distance)
        {
            if (walked != null)
            {
                walked(distance);  //raise the walked event
            }
        }

        public int Age()
        {
            int age = DateTime.Now.Year - mBirthDate.Year; 
            //age can be 1 to high if this year's birthdate is in the future
            if (mBirthDate.AddYears(age) > DateTime.Now)
            {
                age -= 1; //correct age
            }
            return age;
        }

        public DateTime BirthDate
        {
            get
            {
                return mBirthDate;
            }
        }

        public string Name
        {
            get
            {
                return mName;
            }
            set
            {
                if (string.IsNullOrWhiteSpace(value))
                {
                    throw new Exception("Invalid name.");
                }
                mName = value;
            }
        }

        public static int Counter
        {
            get
            {
                return mCounter;
            }
        }

        public static bool CompareAge(Person person1, Person person2)
        {
            return (person1.Age() > person2.Age());
        }

        public static bool CompareAge(object person1, object person2)
        {
            return (((Person)person1).Age() > ((Person)person2).Age());
        }

            // String.Compare compares two specified String objects and returns an integer that 
            // indicates their relative position in the sort order. 
            // < 0 strA is less than strB. 
            // = 0 strA equals strB. 
            // > 0 strA is greater than strB.

        //public static bool CompareName(object person1, object person2)
        //{
        //    if ((String.Compare(((Person)person1).Name, ((Person)person2).Name)) != -1)
        //    {
        //        return true;
        //    }
        //    else
        //    {
        //        return false;
        //    }
        //}

        //andere Schreibweise 1:
        //public static bool CompareName(object person1, object person2)
        //{
        //    if ((String.Compare(((Person)person1).Name, ((Person)person2).Name)) != -1)
        //        return true;
        //    else
        //        return false;
        //}

        //andere Schreibweise 2:
        //public static bool CompareName(object person1, object person2)
        //{
        //    return (String.Compare(((Person)person1).Name, ((Person)person2).Name)) != -1;
        //}

        //andere Schreibweise 3:
        //public static bool CompareName(object person1, object person2)
        //{
        //    return (String.Compare(((Person)person1).Name, ((Person)person2).Name)) != -1 ? true : false;
        //}

        #region IDisposable Members

        public void Dispose()
        {
            throw new NotImplementedException();
        }

        #endregion
    }
}
