﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace NewGenericDelegates
{
    class Program
    {


        static void Main(string[] args)
        {
            List<Student> mStudents = new List<Student>();

            //Action<T>		=> 
            //invoke a delegate on all the items in a collection with a single call.
            //output each student's ToString() info
            mStudents.ForEach(Print);

            //Func<T>		=> 
            //Encapsulates a method that has one parameter and returns a value of the type specified by the TResult parameter.
            //output "Name, Firstname" for all students
            Func<Student, String> convertMethodDelegate;
            convertMethodDelegate = new Func<Student, String>(CanonicalName);
            foreach (var s in mStudents)
            {
                Console.WriteLine(convertMethodDelegate(s));
            }

            //Comparison<T>	=> 
            //Represents the method that compares two objects of the same type.
            mStudents.Sort(BirthDateComparer);

            //Converter<T>	=> 
            //Represents a method that converts an object from one type to another type.
            List<Graduate> graduates;
            graduates = mStudents.ConvertAll(MakeGraduate);

            //Predicate<T>	=> 
            //Represents the method that defines a set of criteria and determines whether the specified object 
            //meets those criteria.
            //check whether as student is of full age (older than 18 years)
            Student firstStudentOfFullAge = mStudents.Find(OfFullAge);
        }

        static void Print(Student s)
        {
            Console.WriteLine(s.ToString());
        }

        static String CanonicalName(Student s)
        {
            return (String.Format("{0}, {1}",s.Name, s.FirstName));
        }

        static int BirthDateComparer(Student s1, Student s2)
        {
            if (s1.BirthDate < s2.BirthDate)
            {
                return -1;
            }
            else if (s1.BirthDate > s2.BirthDate)
            {
                return 1;
            }
            else
            {
                return 0;
            }

        }

        static Graduate MakeGraduate(Student s)
        {
            return (new Graduate(s.Gender, s.Name, s.FirstName, s.BirthDate, DateTime.Now, 1));
        }

        static bool OfFullAge(Student s)
        {
            if ((DateTime.Now.Year - s.BirthDate.Year) > 18)
            {
                return true;
            }
            else
            {
                return false;
            }
        }
    }
}
