﻿using Microsoft.VisualBasic;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Text;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using System.Xml.Serialization;

[Serializable()]
public class School
{

	private List<Department> mDepartments = new List<Department>();
	public List<Department> Departments
	{
		get { return mDepartments; }
		set { mDepartments = value; }
	}


	private static int mCount = 0;

	private int mID;
	public int ID
	{
		get { return mID; }
	}


	private string mName;
	public string Name
	{
		get { return mName; }
		set { mName = value; }
	}

	public School(string name)
	{
		mID = mCount;
		mCount += 1;
		mName = name;
	}

	//for XML serialization
	private School()
	{

	}

	public override string ToString()
	{
		StringBuilder sb = new StringBuilder();
		//sb.Append("School object model by HMF" & vbCrLf)
		sb.Append("School ID: " + mID + ", Name: " + mName + ", Nr. of Departments: " + mDepartments.Count);
		foreach (Department d in mDepartments)
		{
			sb.Append("\r\n\t" + d.ToString());
		}
		return sb.ToString();
	}

	public void SortStudentsByGender()
	{
		foreach (Department d in mDepartments)
		{
			d.SortStudentsByGender();
		}
	}

	//Sternzeichen
	public void SortStudentsBySunSign()
	{
		foreach (Department d in mDepartments)
		{
			d.SortStudentsBySunSign();
		}
	}

	public void SortStudentsByBirthdate()
	{
		foreach (Department d in mDepartments)
		{
			d.SortStudentsByBirthdate();
		}
	}

	#region "Binary Serialization"
	public void SerializeBinary(string file)
	{
		BinaryFormatter form = new BinaryFormatter();
		FileStream stream = null;
		stream = new FileStream(file, FileMode.Create, FileAccess.Write, FileShare.None);
		form.Serialize(stream, this);
		stream.Close();
	}

	public static School DeserializeBinary(string file)
	{
		School s = null;
		BinaryFormatter form = new BinaryFormatter();
		FileStream stream = null;
		stream = new FileStream(file, FileMode.Open, FileAccess.Read, FileShare.Read);
		s = (School) form.Deserialize(stream);
		stream.Close();
		return s;
	}
	#endregion

	#region "XML Serialization"
	public void SerializeXML(string file)
	{
		XmlSerializer serializer = new XmlSerializer(this.GetType());
		StreamWriter writer = new StreamWriter(file);
		serializer.Serialize(writer, this);
		writer.Close();
	}

	public static School DeserializeXML(string file)
	{
		School s = null;
		StreamReader reader = new StreamReader(file);
		XmlSerializer serializer = new XmlSerializer(typeof(School));
		s = (School) serializer.Deserialize(reader);
		reader.Close();
		return s;
	}
	#endregion

	#region "Create test data htl-donaustadt"
	public static School CreateSchoolTestData()
	{
		//create students for 5ADH 
		SchoolClass sc1 = new SchoolClass("5ADH", "D200");
		sc1.Students.Add(new Student(Genders.Female, "B", "P", new DateTime(1989,9,4)));
		sc1.Students.Add(new Student(Genders.Male, "B", "D", new DateTime(1987,9,20)));
		sc1.Students.Add(new Student(Genders.Male, "B", "C", new DateTime(1988,9,24)));
		sc1.Students.Add(new Student(Genders.Male, "G", "S", new DateTime(1987,8,1)));
		sc1.Students.Add(new Student(Genders.Male, "H", "F", new DateTime(1987,8,23)));
		sc1.Students.Add(new Student(Genders.Male, "L", "M", new DateTime(1989,11,21)));
		sc1.Students.Add(new Student(Genders.Male, "P", "M", new DateTime(1989,12,4)));
		sc1.Students.Add(new Student(Genders.Male, "S", "A", new DateTime(1989,12,11)));
		sc1.Students.Add(new Student(Genders.Male, "S", "E", new DateTime(1990,7,31)));
		sc1.Students.Add(new Student(Genders.Male, "V", "S", new DateTime(1988,5,19)));
		sc1.Students.Add(new Student(Genders.Female, "V", "M", new DateTime(1989,10,29)));
		sc1.Students.Add(new Student(Genders.Male, "V", "M", new DateTime(1990,5,26)));
		sc1.Students.Add(new Student(Genders.Male, "W", "P", new DateTime(1989,7,21)));
		sc1.Students.Add(new Student(Genders.Male, "W", "T", new DateTime(1987,11,30)));

		//create students for 5BDH
		SchoolClass sc2 = new SchoolClass("5BDH", "D200");
		sc2.Students.Add(new Student(Genders.Male, "B", "K", new DateTime(1990,5,3)));
		sc2.Students.Add(new Student(Genders.Male, "B", "M", new DateTime(1990,7,7)));
		sc2.Students.Add(new Student(Genders.Male, "B", "A", new DateTime(1988,8,8)));
		sc2.Students.Add(new Student(Genders.Male, "C", "A", new DateTime(1990,3,6)));
		sc2.Students.Add(new Student(Genders.Female, "D", "Ö", new DateTime(1988,10,18)));
		sc2.Students.Add(new Student(Genders.Male, "E", "M", new DateTime(1989,12,5)));
		sc2.Students.Add(new Student(Genders.Male, "F", "T", new DateTime(1990,1,28)));
		sc2.Students.Add(new Student(Genders.Male, "F", "R", new DateTime(1989,8,12)));
		sc2.Students.Add(new Student(Genders.Male, "K", "C", new DateTime(1990,8,31)));
		sc2.Students.Add(new Student(Genders.Female, "L", "V", new DateTime(1990,3,9)));
		sc2.Students.Add(new Student(Genders.Male, "M", "R", new DateTime(1988,11,16)));
		sc2.Students.Add(new Student(Genders.Male, "N", "L", new DateTime(1990,4,4)));
		sc2.Students.Add(new Student(Genders.Male, "P", "D", new DateTime(1986,8,25)));
		sc2.Students.Add(new Student(Genders.Female, "S", "C", new DateTime(1990,2,17)));
		sc2.Students.Add(new Student(Genders.Female, "S", "N", new DateTime(1988,2,14)));
		sc2.Students.Add(new Student(Genders.Male, "S", "A", new DateTime(1990,3,19)));
		sc2.Students.Add(new Student(Genders.Male, "W", "D", new DateTime(1989,10,28)));

		//create students for 5EDH 
		SchoolClass sc3 = new SchoolClass("5EDH", "D200");
		sc3.Students.Add(new Student(Genders.Male, "B", "R", new DateTime(1989,09,18)));
		sc3.Students.Add(new Student(Genders.Male, "B", "B", new DateTime(1990,8,24)));
		sc3.Students.Add(new Student(Genders.Male, "D", "A", new DateTime(1990,7,4)));
		sc3.Students.Add(new Student(Genders.Male, "D", "A", new DateTime(1989,8,29)));
		sc3.Students.Add(new Student(Genders.Male, "G", "S", new DateTime(1989,10,6)));
		sc3.Students.Add(new Student(Genders.Male, "G", "D", new DateTime(1990,6,13)));
		sc3.Students.Add(new Student(Genders.Male, "G", "M", new DateTime(1989,9,6)));
		sc3.Students.Add(new Student(Genders.Male, "H", "M", new DateTime(1990,3,4)));
		sc3.Students.Add(new Student(Genders.Male, "H", "P", new DateTime(1988,2,22)));
		sc3.Students.Add(new Student(Genders.Male, "K", "P", new DateTime(1990,1,19)));
		sc3.Students.Add(new Student(Genders.Male, "L", "C", new DateTime(1989,11,26)));
		sc3.Students.Add(new Student(Genders.Male, "S", "D", new DateTime(1990,5,4)));
		sc3.Students.Add(new Student(Genders.Male, "S", "A", new DateTime(1989,9,5)));
		sc3.Students.Add(new Student(Genders.Male, "S", "P", new DateTime(1989,8,28)));
		sc3.Students.Add(new Student(Genders.Male, "T", "S", new DateTime(1988,7,9)));
		sc3.Students.Add(new Student(Genders.Male, "T", "P", new DateTime(1989,7,1)));
		sc3.Students.Add(new Student(Genders.Male, "V", "J", new DateTime(1989,4,6)));

		//create students for 4ADH 
		SchoolClass sc4 = new SchoolClass("4ADH", "D200");
		sc4.Students.Add(new Student(Genders.Male, "A", "J", new DateTime(1990,3,24)));
		sc4.Students.Add(new Student(Genders.Female, "A", "L", new DateTime(1989,11,25)));
		sc4.Students.Add(new Student(Genders.Male, "C", "B", new DateTime(1989,10,9)));
		sc4.Students.Add(new Student(Genders.Female, "D", "K", new DateTime(1991,4,28)));
		sc4.Students.Add(new Student(Genders.Male, "E", "R", new DateTime(1990,11,2)));
		sc4.Students.Add(new Student(Genders.Female, "H", "S", new DateTime(1991,1,11)));
		sc4.Students.Add(new Student(Genders.Male, "H", "M", new DateTime(1986,1,15)));
		sc4.Students.Add(new Student(Genders.Male, "J", "R", new DateTime(1990,12,13)));
		sc4.Students.Add(new Student(Genders.Male, "K", "P", new DateTime(1991,4,1)));
		sc4.Students.Add(new Student(Genders.Male, "K", "Y", new DateTime(1988,11,2)));
		sc4.Students.Add(new Student(Genders.Male, "K", "C", new DateTime(1991,5,5)));
		sc4.Students.Add(new Student(Genders.Female, "L", "P", new DateTime(1989,10,17)));
		sc4.Students.Add(new Student(Genders.Male, "M", "R", new DateTime(1991,9,4)));
		sc4.Students.Add(new Student(Genders.Male, "N", "P", new DateTime(1991,3,29)));
		sc4.Students.Add(new Student(Genders.Male, "O", "P", new DateTime(1991,6,10)));
		sc4.Students.Add(new Student(Genders.Male, "P", "M", new DateTime(1991,5,4)));
		sc4.Students.Add(new Student(Genders.Male, "S", "S", new DateTime(1991,5,28)));
		sc4.Students.Add(new Student(Genders.Male, "S", "C", new DateTime(1991,6,8)));
		sc4.Students.Add(new Student(Genders.Male, "S", "Z", new DateTime(1990,6,11)));
		sc4.Students.Add(new Student(Genders.Male, "S", "A", new DateTime(1991,4,26)));
		sc4.Students.Add(new Student(Genders.Male, "S", "C", new DateTime(1991,4,3)));
		sc4.Students.Add(new Student(Genders.Female, "S", "H", new DateTime(1990,7,4)));
		sc4.Students.Add(new Student(Genders.Male, "T", "M", new DateTime(1990,10,1)));
		sc4.Students.Add(new Student(Genders.Male, "V", "", new DateTime(1991,6,8)));

		//create students for 4BDH
		SchoolClass sc5 = new SchoolClass("4BDH", "D200");

		//create students for 4EDH 
		SchoolClass sc6 = new SchoolClass("4EDH", "D200");

		//create department
		Department d1 = new Department("EDV-O");
		d1.SchoolClasses.Add(sc1);
		d1.SchoolClasses.Add(sc2);
		d1.SchoolClasses.Add(sc3);
		d1.SchoolClasses.Add(sc4);
		d1.SchoolClasses.Add(sc5);
		d1.SchoolClasses.Add(sc6);

		Department d2 = new Department("Elektronik");
		Department d3 = new Department("Abendschule");

		//create school
		School s = new School("htl-donaustadt");
		s.Departments.Add(d1);
		s.Departments.Add(d2);
		s.Departments.Add(d3);

		return s;
	}
	#endregion

}
