﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CalculateDelegate
{
    class Program
    {
        delegate int CalcDelegate(int num);

        static void Main(string[] args)
        {
            //old style:
            Calculate1(5, new CalcDelegate(Square));

            //Lambda expressions are inline functions:
            //compiler generates anonymous delegate:
            Calculate1(5, num => num * num);

            //.NET 3.5 includes the System.Func generic delegate.
            //it has five signatures, that all include the data 
            //type of the returned value but vary depending on 
            //the number of arguments supported, which ranges 
            //from 0 to 5.
            Func<int, int> calc2 = num => num * num;
            Calculate2(5, calc2);

            Console.ReadLine();
        }

        //create the square of an integer value
        static int Square(int num)
        {
            return num * num;
        }

        //execute a calculation given by a delegate parameter 
        //on an integer value  
        static void Calculate1(int num, CalcDelegate calculation)
        {
            Console.WriteLine(calculation(num).ToString());
        }

        //execute a calculation given by a delegate parameter 
        //on an integer value using the generic Delegate Func, 
        //saves the definition of a delegate
        static void Calculate2(int num, Func<int, int> calculation)
        {
            Console.WriteLine(calculation(num).ToString());
        }
    }
}
