﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace FileWatcher
{
    class Program
    {
        static void Main(string[] args)
        {
            const string DIRECTORY = @"C:\temp";     //directory to watch

            FileSystemWatcher fsw = new FileSystemWatcher(DIRECTORY);

            //wire up event handlers
            fsw.Changed += new FileSystemEventHandler(fsw_Changed); 
            fsw.Created += new FileSystemEventHandler(fsw_Changed); 
            fsw.Deleted += new FileSystemEventHandler(fsw_Changed);
            fsw.Renamed += new RenamedEventHandler(fsw_Renamed);
            fsw.Error += new ErrorEventHandler(fsw_Error); //internal buffer overflow

            //limit the amount of information to avoid buffer overflow errors
            //Watch for changes in LastAccess and LastWrite times, and the renaming of files or directories
            fsw.NotifyFilter = (NotifyFilters.LastAccess | NotifyFilters.LastWrite
                                | NotifyFilters.FileName | NotifyFilters.DirectoryName); 
            fsw.Filter = "*.txt";
            fsw.IncludeSubdirectories = false;

            //start the surveillance
            fsw.EnableRaisingEvents = true;

            Console.WriteLine("Hit Enter to exit.");
            Console.ReadLine();
        }

        static void fsw_Changed(object source, FileSystemEventArgs e)
        {
            Console.WriteLine("File " + e.FullPath + " " + e.ChangeType.ToString());
        }

        static void fsw_Renamed(object source, RenamedEventArgs e)
        {
            Console.WriteLine("File " + e.OldFullPath + " renamed to " + e.FullPath);
        }

        static void fsw_Error(object source, ErrorEventArgs e)
        {
            Console.WriteLine("Error with File " + e.GetException().ToString());
        }

    }
}
